;==============================================================================
;File:      Header_Keyboard.asm
;Version:   2.0 - Dec/04/2013
;Author:    TK90X Fan (reikainosuke-tk@yahoo.com.br)
;Description:
; Set of constants for TK90X Z80 assembler. This set contains constants for 
;keyboard.
;License:
;    This program is free software: you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation, either version 3 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program.  If not, see <http://www.gnu.org/licenses/>.
;
;==============================================================================
;
;==============================================================================
; Constants
;==============================================================================
; Key codes returned by KEY_SCAN ROM routine.
;------------------------------------------------------------------------------
KCODE_B         EQU 0
KCODE_H         EQU 1
KCODE_Y         EQU 2
KCODE_6         EQU 3
KCODE_5         EQU 4
KCODE_T         EQU 5
KCODE_G         EQU 6
KCODE_V         EQU 7
KCODE_N         EQU 8
KCODE_J         EQU 9
KCODE_U         EQU 10
KCODE_7         EQU 11
KCODE_4         EQU 12
KCODE_R         EQU 13
KCODE_F         EQU 14
KCODE_C         EQU 15
KCODE_M         EQU 16
KCODE_K         EQU 17
KCODE_I         EQU 18
KCODE_8         EQU 19
KCODE_3         EQU 20
KCODE_E         EQU 21
KCODE_D         EQU 22
KCODE_X         EQU 23
KCODE_SShift    EQU 24
KCODE_L         EQU 25
KCODE_O         EQU 26
KCODE_9         EQU 27
KCODE_2         EQU 28
KCODE_W         EQU 29
KCODE_S         EQU 30
KCODE_Z         EQU 31
KCODE_Space     EQU 32
KCODE_Enter     EQU 33
KCODE_P         EQU 34
KCODE_0         EQU 35
KCODE_1         EQU 36
KCODE_Q         EQU 37
KCODE_A         EQU 38
KCODE_CShift    EQU 39
;
;------------------------------------------------------------------------------
; Keyboard I/O hardware port reading
; The ZX Spectrum keyboard is composed by a matrix of 40 keys, arranged in 8 
;rows of 5 keys each and 5 columns of 8 keys each. 
; The least signficant byte of port address is given by KBD_PORT. The port can
;be read by LD C,KBD_PORT and IN R,(C) or LD A,(KBD_PORT).
; The most significant byte enables zero or more rows to be read. Loading B with
;KROW_1L, for example, enables upper left row of keys to be read for IN R,(C)
;instruction. More rows may be enabled if mixed by AND binary operator. 
; The resulting value is stored into bits 0-4. 
;------------------------------------------------------------------------------
;
KBD_PORT        EQU 254         ; Least significant byte or keyboard/border/mic/
                                ;sound I/O port. 
; The following values enable one row with 5 keys. The rows are numbered from 
;top to bottom (0-3) and split em right and left set of 5 keys. More than one
;row can be enabled making AND operation with each desired row (e.g. 
;LD B,KROW_1L&KROW_1R). 
; 
KROW_0L         EQU %11110111   ; Enable 1, 2, 3, 4, 5. 
KROW_1L         EQU %11111011   ; Enable Q, W, E, R, T. 
KROW_2L         EQU %11111101   ; Enable A, S, D, F, G. 
KROW_3L         EQU %11111110   ; Enable Caps-shift, Z, X, C, V. 
KROW_0R         EQU %11101111   ; Enable 6, 7, 8, 9, 0. 
KROW_1R         EQU %11011111   ; Enable Y, U, I, O, P. 
KROW_2R         EQU %10111111   ; Enable H, J, K, L, Enter. 
KROW_3R         EQU %01111111   ; Enable B, N, M, Symbol-shift, Space.
;
; A key of a column is probed by making AND operation with KCOL_x and the value
;read from I/O port (the exact key depends on enabled row). The following 
;example reads Enter and Space keys and return 0 if neither has been pressed.
;   LD A,KROW_2R&KROW_2R         Enable rows containing Enter and Space.
;   IN A,(KBD_PORT)              Read keyboard I/O port.
;   AND KCOL_3                   Probe column containing Enter and Space.
;
KCOL_0          EQU %00000001   ; Probe 1, Q, A, Caps-shift, 6, Y, H, B.
KCOL_1          EQU %00000010   ; Probe 2, W, S, Z, 7, U, J, N.
KCOL_2          EQU %00000100   ; Probe 3, E, D, X, 8, I, K, M. 
KCOL_3          EQU %00001000   ; Probe 4, R, F, C, 9, O, L, Symbol-shift.
KCOL_4          EQU %00010000   ; Probe 5, T, G, V, 0, P, Enter, Space. 
;
;==============================================================================
; ROM routine addresses
;==============================================================================
;
KEY_SCAN:       EQU 654         ; Read keyboard and return in DE registers:
                                ;E=key code or 255 if nothing was pressed;
                                ;D=255 if only one key was pressed, otherwise
                                ;the code of Caps-shift or Symbol-shift key
                                ;pressed (the Caps-shift takes precedence);
                                ;Z flag is reset if many keys with combination 
                                ;other than shift+key were pressed.
BREAK_KEY:      EQU 8020        ; Return flag C=0 if keys shift and space
                                ;(break) were pressed.
PAUSE:          EQU 7994        ; Same as BASIC. Operand is taken from top of 
                                ;stack of BASIC calculator. 
PAUSE_1:        EQU 7997        ; Same as PAUSE but operand must be in BC 
                                ;register pair.
;
;==============================================================================
; Sinclair BASIC system variables
;==============================================================================
;
KSTATE:         EQU 23552       ; Used by keyboard scan.
LASTK:          EQU 23560       ; Code of last key pressed.

