;===============================================================================
; Program:  Digi-Pop
; Part 3:   Hypervelocity Stars
; Code:     TK90X Fan
; Song&gfx: Mister Beep
; Date:     Feb/04/2014
;===============================================================================
; Constants
;-------------------------------------------------------------------------------
MusicData       EQU 29280       ; Start of music data. 
BeplayerRun     EQU 64000       ; Start of music player. 
ScrollAttrib    EQU 64+8*1+5    ; BRIGHT + PAPER 1 + INK 5.
ScrolVideo      EQU 16384       ; Start position of video RAM to print scrolling
                                ;message (line 0, column 0). 
ScrolAttMem     EQU 22528       ; Start position of attribute area for scroll.
;
    INCLUDE "Header_Keyboard.asm"
;
;===============================================================================
; Start of program
;-------------------------------------------------------------------------------
    ORG 25088
StartProg
    JP MainProg
;
;===============================================================================
; Variables
;------------------------------------------------------------------------------
ScrSpeed        DEFB 2          ; Speed of scrolling (1=stopped, 2=slowest, 
                                ;5=fastest). 
TxtPtr          DEFW Text       ; Current address of scrolling text.
NumPix          DEFB 1          ; Counter of pixels of character being scrolled. 
ChrModel        DEFS 8          ; Bitmap of current character (8 bytes). 
;
;===============================================================================
; Routines and subroutines
;-------------------------------------------------------------------------------
MainProg
;-------------------------------------------------------------------------------
    CALL InitPart
    LD B,150
Delay
    HALT
    DJNZ Delay
    CALL RunPart
    LD A,7
    OUT (254),A
    LD HL,22528
    LD (HL),255
    LD DE,22529
    LD BC,767
    LDIR
    LD B,50
Delay2
    HALT
    DJNZ Delay2
    RST 0
;
;-------------------------------------------------------------------------------
InitPart
;-------------------------------------------------------------------------------
; Move and decompress music data and player. 
    LD HL,MusicEnd-1            ; Move compressed data to top of RAM. 
    LD DE,65535
    LD BC,MusicEnd-MusicZx7
    LDDR
    INC DE
    EX DE,HL                    ; HL=start of compressed data.
    LD DE,MusicData             ; DE=start of uncompressed data. 
    CALL dzx7
    LD HL,BeplayerZx7           ; Decompress compressed music player. 
    LD DE,BeplayerRun
    CALL dzx7
; Save first line of screen (line 0) data to buffer. This line will be 
;overwritten by the scroller, hence must be preserved. 
    LD HL,ScrolVideo            ; Copy screen line 0 data to buffer. 
    LD DE,LineBuffer
    LD A,8                      ; 8 pixel lines must be copied. 
InitP.l0
    PUSH HL
    LD BC,32                    ; One line is 32 bytes long. 
    LDIR
    POP HL
    INC H                       ; Calculate next line address. 
    DEC A
    JR NZ,InitP.l0
    LD HL,ScrolAttMem           ; Copy color attributes of line 0 to buffer. 
    LD C,32
    LDIR
    RET
;
;-------------------------------------------------------------------------------
; Decompressor for ZX7 compressed files. Code by Einar Saukas.
; Available at: http://www.worldofspectrum.org/infoseekid.cgi?id=0027996
;-------------------------------------------------------------------------------
    INCLUDE "dzx7_mega.asm"
;
;-------------------------------------------------------------------------------
RunPart
;-------------------------------------------------------------------------------
    LD HL,ScrolVideo            ; Clear first line of screen. 
    LD C,8                      ; 8 pixel lines must be cleared. 
RunPart.l0
    LD B,32                     ; Number of bytes in one line. 
    PUSH HL
RunPart.l1
    LD (HL),0
    INC HL
    DJNZ RunPart.l1
    POP HL
    INC H
    DEC C
    JR NZ,RunPart.l0
    LD B,32                     ; Define attributes of first line of screen. 
    LD HL,ScrolAttMem
RunPart.l2
    LD (HL),ScrollAttrib
    INC HL
    DJNZ RunPart.l2
RunPart.l3
    CALL Scroller               ; Call scroller routine. 
    HALT                        ; Wait for next interrupt. 
    CALL KEY_SCAN               ; Read keyboard. 
    JR NZ,RunPart.l3            ; Read keyboard again if illegal key pressed. 
    LD A,E
    CP KCODE_Q                  ; Finish routine execution if "Q" was pressed. 
    JR NZ,RunPart.s0
    CALL RestoreScr
    RET
RunPart.s0
    CP KCODE_M
    JR NZ,RunPart.l3            ; Read keyboard again if key other than "M" was
                                ;pressed. 
    CALL PlaySong               ; Play song. 
    JR RunPart                  ; Repeat all the loop. 
;
;-------------------------------------------------------------------------------
RestoreScr
;-------------------------------------------------------------------------------
    LD HL,LineBuffer            ; Copy buffer data to screen line 0. 
    LD DE,ScrolVideo
    LD A,8                      ; 8 pixel lines must be copied. 
RestoreScr.l0
    PUSH DE
    LD BC,32                    ; One line is 32 bytes long. 
    LDIR
    POP DE
    INC D                       ; Calculate next line address. 
    DEC A
    JR NZ,RestoreScr.l0
    LD DE,ScrolAttMem           ; Copy color attributes of line 0 from buffer. 
    LD C,32
    LDIR
    RET
;
;-------------------------------------------------------------------------------
PlaySong
;-------------------------------------------------------------------------------
    PUSH AF
    CALL RestoreScr
    CALL BeplayerRun            ; Execute music player routine. 
    LD HL,Text                  ; Initialize scroll text pointer and pixel
    LD (TxtPtr),HL              ;counter before returning. 
    LD HL,NumPix
    LD (HL),1
    POP AF
    RET
;
;-------------------------------------------------------------------------------
Scroller
;-------------------------------------------------------------------------------
    LD A,(ScrSpeed)             ; Get scroller speed. 
Scroller.loop
    DEC A                       ; Decrement speed counter and finish when 0. 
    RET Z
    PUSH AF                     ; Save speed counter.
    CALL Scroller.main          ; Execute main scroller routine. 
    POP AF                      ; Restore speed counter. 
    JR Scroller.loop            ; Repeat Scroller routine.
;
;------------------------------------------------------------------------------
Scroller.main
;------------------------------------------------------------------------------
    LD HL,NumPix                ; Decrement counter of pixels. 
    DEC (HL)
    JR NZ,Scroller.SkIniChr     ; Jump if result is greater than 0. 
    LD (HL),8                   ; Initialize counter to 8. 
    LD DE,ScrolAttMem           ; Scroll color attributes to left. 
    LD HL,ScrolAttMem+1
    LD BC,31
    LDIR
    DEC HL
    LD (HL),ScrollAttrib        ; Define rightmost character color attribute. 
    LD HL,(TxtPtr)
    INC HL                      ; Advance text pointer one position. 
    LD (TxtPtr),HL
    LD A,(HL)                   ; Take character from text. 
    CP " "                      ; Jump if character is greater than space. 
    JR NC,Scroller.SkIniTxtP
    LD HL,Text                  ; Initialize pointer to start of text. 
    LD (TxtPtr),HL
    JR Scroller.main            ; Jump to print the first character of text. 
Scroller.SkIniTxtP
    LD H,0                      ; Calculate address of character bitmap given 
    LD L,A                      ;by: CharBitmap+8*A-256. The factor 8 for A is 
    ADD HL,HL                   ;the size of bitmap of one character (8 bytes). 
    ADD HL,HL                   ;Since first character code is 32 (space), an 
    ADD HL,HL                   ;offset of -256 is applied (32*8=256).
    LD DE,CharBitmap-256
    ADD HL,DE
    LD DE,ChrModel 
    LD B,8                      ; Counter for 8 bytes of character bitmap. 
Scroller.l0
    LD A,(HL)                   ; Copy byte from bitmap to character model. 
    LD (DE),A
    INC HL                      ; Increment pointers. 
    INC DE 
    DJNZ Scroller.l0            ; Repeat until all 8 bytes were copied. 
Scroller.SkIniChr
;
;------------------------------------------------------------------------------
; Scroll Text Line
; Shift to left 1 text line, composed by 8 vertical pixels and 32 horizontal 
;columns of 256 pixels. 
;------------------------------------------------------------------------------
ScrTxtLin
    LD DE,ChrModel              ; DE is pointer for invisible character buffer.
    LD B,8                      ; Counter for 8 lines. 
    LD HL,ScrolVideo+31         ; HL points to video RAM, at line 0&col. 31. 
ScrTxtLin.l0
    CALL ScrPixLin              ; Scroll the line 1 bit to left. 
    DJNZ ScrTxtLin.l0           ; Repeat until all 8 lines were scrolled. 
    RET   
;
;------------------------------------------------------------------------------
; Scroll Pixel Line
; Shift to left pixel line composed by 32 columns (256 pixels). 
; Input: 
;   HL=address of right limit of line
;   DE=address of invisible character in the buffer
; Used registers: AF, BC, DE, HL.  
;------------------------------------------------------------------------------
ScrPixLin
    PUSH HL
    LD A,(DE)
    RLCA
    LD (DE),A
    LD C,32
ScrPixLin.l0
    RL (HL)
    DEC HL
    DEC C
    JR NZ,ScrPixLin.l0
    POP HL
    INC H
    INC DE
    RET   
;
;===============================================================================
; Data
;------------------------------------------------------------------------------
CharBitmap 
;
; Bitmap data of Data 70 font. Taken from ZX-Alfa (Einar Saukas and Eduardo Ito)
;at http://www.worldofspectrum.org/infoseekid.cgi?id=0025245.
;------------------------------------------------------------------------------

    INCBIN "ZXAlfa_Data70.bin"
;
;------------------------------------------------------------------------------
Text
;
; Text for the scroller. 
;------------------------------------------------------------------------------
    DEFM " "
    INCBIN "ScrollPart3.txt"
    DEFM "        "
    DEFB 0

;
;------------------------------------------------------------------------------
LineBuffer
;
; Buffer to save screen line. It'll overwrite BeplayerZx7.  
;
BeplayerZx7
;
; Compressed machine code of music player. 
;------------------------------------------------------------------------------
    INCBIN "BEPLAYER_64000_1536.bin.zx7"
;
;------------------------------------------------------------------------------
MusicZx7
;------------------------------------------------------------------------------
    INCBIN "STARS_29280_34208.bin.zx7"
MusicEnd
    END


